function [Freq_shift_lower, Freq_shift_upper] = abs_dist_cal(SMI_Matrix, Samp_Rate, L_vec)
%% Pablo Garca - UQ 2017
% INPUT: 
% SMI_Matrix = Signals from the SMI (Matrix per columns)
% Samp_Rate = Sampling rate (samples/time) 
% L_vec = Distance of the target (column vector)
% OUTPUT:
% Frequency shift limits

%% Set parameters
N_points = size(SMI_Matrix,2); % Number of columns introduced
m_vec = zeros(N_points,1); % Number of fringes per distance
c = 3E8; % Light velocity
next = 1; % Refraction index

for jj = 1:N_points
    m_vec(jj) = abs_dist_fringes(SMI_Matrix(:,jj),Samp_Rate);  
end

figure('Name','Fringes vs Distance')
plot(L_vec,m_vec,'*b')
xlabel('L (m)')
ylabel('Number of fringes')

%% Fit 
Freq_shift_lower = L_vec\floor(m_vec-1); Freq_shift_upper = L_vec\floor(m_vec+1);
Freq_shift_lower = Freq_shift_lower*c/(2*next);
Freq_shift_upper = Freq_shift_upper*c/(2*next);

hold on
plot((0:0.001:0.5),floor(2*(0:0.001:0.5)*Freq_shift_upper/3E8)+1,'--b')
plot((0:0.001:0.5),floor(2*(0:0.001:0.5)*Freq_shift_lower/3E8)-1,'--b')
plot((0:0.001:0.5),floor(2*(0:0.001:0.5)*...
    (Freq_shift_lower+Freq_shift_upper)*0.5/3E8),'-r')
legend('Experimental fringes', 'Floor(Nf + 1)', 'Floor(Nf - 1)', 'Nf', 'Location', 'best')
hold off

figure('Name','Distances')
plot(L_vec,m_vec*c/(2*0.5*(Freq_shift_upper+Freq_shift_lower)),'*r')
hold on
plot(L_vec,L_vec,'-b')
hold off
legend('Measured distance', 'Reference y = x', 'Location', 'best')
xlabel('Distance(m)'); ylabel('Measured distance(m)');
end