function m = abs_dist_fringes(SMI_signal,Samp_Rate) 
%% Pablo Garca - UQ 2017
% INPUT: 
% SMI_signal = Signal from the SMI (column vector)
% Sampling_Rate = Sampling rate (samples/time) 
% OUTPUT:
% m = Total number of fringes

%% Set parameters
threshold = 1.0; % Set a threshold for a peak in 100% mean
h_der = 1/Samp_Rate; % Time step difference

%% Filter data 
Window_Size = round(3E-5*Samp_Rate); % Create the filtering window
b = (1/Window_Size)*ones(1,Window_Size); a = [1]; % Moving mean filter 
SMI_filter = filter(b,a,SMI_signal); 

figure(1)
plot(SMI_signal,'-r'); hold on; plot(SMI_filter,'-b');

%% Differenciate the signal 
% 1st FW der from the filtered signal; BC: S_derv(t=T) = S_derv(t=T-h)
S_derv = (1/h_der)*(SMI_filter(2:end)-SMI_filter(1:end-1)); 
S_derv = [S_derv; S_derv(end)];

ind = find(S_derv(2:end).*S_derv(1:end-1) < 0); 
if isempty(ind) == 0
    [~,first_ind] = max([ind(2)-ind(1); ind(3)-ind(2)]);
    S_derv_window = S_derv(ind(first_ind):ind(first_ind + 1));
    slope = mean(S_derv_window); 
    SMI_removed = SMI_filter(ind(first_ind):ind(first_ind + 1)) + ...
        max(SMI_filter(ind(first_ind):ind(first_ind + 1))) -...
        slope*(linspace(0,length(S_derv_window),length(S_derv_window)))'*h_der;
    
    figure(2); 
    subplot(2,2,1); plot(SMI_filter); hold on; plot(SMI_signal,'--');hold off;
    subplot(2,2,2); plot(S_derv); 
    subplot(2,2,3); plot(SMI_removed);
    subplot(2,2,4); plot(S_derv(ind(first_ind):ind(first_ind + 1))); 
    
    %% Normalize the signal and find the transitions
    S_derv_norm = S_derv_window/max(abs(S_derv_window)); % Normalize the SMI signal
    S_derv_norm = filter(b,a,S_derv_norm);
    % Find all values position below the threshold
    [~,pos] = findpeaks(SMI_removed); m = length(pos);

    figure(3)
    plot(S_derv_norm,'--r'); hold on; 
    plot(pos,S_derv_norm(pos),'*b');
    line([0 length(S_derv_norm)],[mean(S_derv_norm)*threshold...
        mean(S_derv_norm)*threshold]); 
    hold off;

else
    m = 0; 
end

end
