clear all
close all

%% Parameters
threshold = 0.1; % Percentage above the baseline
t_distance = 5E-3; % Time of error
load_data = 1;
Oil_pump = 30/60; Water_pump = 10/60; % uL/min to uL/s
excen0 = sqrt(Water_pump/Oil_pump); % initial guess for excentricity
Channel_width = 1E3; Channel_depth = 100; % um
Fluid_vel = (Oil_pump + Water_pump)*1E9/(Channel_width * Channel_depth); % um/s

%% Load data
if load_data == 0
    Loaded_data = load('10seconds_600k_Oil_30_Water_10_video.lvm');
    Time = Loaded_data(:,1);
    Data = Loaded_data(:,2);
    save('Bubbles_10s_600k_Oil_30_Water_10_video','Time','Data')
else
    load('Bubbles_10s_600k')
    % load('Bubbles_10s_600k_Oil_10_Water_1')
    % load('Bubbles_10s_600k_Oil_30_Water_10_video')
end

F_sampling = 1/(Time(2) - Time(1)); N_samples = length(Data);

%% Smooth the signal
figure('Name','Smoothed data')
plot(Time,Data,'-b');
xlabel('Time (s)')
ylabel('Power variation')

Window_Size = round(0.005*F_sampling); % Create the filtering window of 5ms
b = (1/Window_Size)*ones(1,Window_Size); a = [1]; % Moving mean filter 
Data = filter(b,a,Data); Data = Data(Window_Size/2+1:end-Window_Size/2);
    
Time = Time(1:end-Window_Size); N_samples = N_samples - Window_Size;

hold on; plot(Time,Data,'-r'); hold off
xlabel('Time (s)')
ylabel('Power variation')
legend('Original','Smoothed')

%% Find peaks
Data = Data - min(Data); Data = Data/max(Data); 
Digital_data = zeros(N_samples,1); 
Digital_data(Data > threshold) = 1; 
Digital_data_prev = Digital_data;

%% Bubbles volume by deriving the digital signal
Digital_data_derv = F_sampling*(Digital_data(2:end)-Digital_data(1:end-1));
[ini_bubble, ~] = find(Digital_data_derv > 0); 
[end_bubble, ~] = find(Digital_data_derv < 0);
cross_min = zeros(length(ini_bubble),1); cross_max = cross_min;
loc_min = cross_min; loc_max = cross_min; loc_mean = cross_min;
frame = round(0.05*F_sampling);

for ii=1:length(ini_bubble)-1
     [cross_min(ii), loc_min(ii)] = min(Data(end_bubble(ii):ini_bubble(ii+1)));
     loc_min(ii) = loc_min(ii) + end_bubble(ii);
     [cross_max(ii), loc_max(ii)] = max(Data(loc_min(ii)-frame:loc_min(ii)+frame));
     loc_max(ii) = loc_min(ii) - frame + loc_max(ii);
     loc_mean(ii) = round((loc_max(ii)+loc_min(ii))/2);
     Digital_data(end_bubble(ii):loc_mean(ii)) = 1;
end
[cross_min(end), loc_min(end)] = min(Data(end_bubble(end):end));
loc_min(end) = loc_min(end) + end_bubble(end);
[cross_min(end), loc_max(end)] = max(Data(loc_min(end)-frame:loc_min(end)+frame));
loc_max(end) = loc_max(end) + loc_min(end) - frame;
loc_mean(end) = round((loc_max(end)+loc_min(end))/2);
Digital_data(end_bubble(end):loc_mean(end)) = 1;

figure('Name','Check')
plot(Data,'-b'); hold on; 
plot(loc_min,cross_min,'*r'); 
plot(loc_max,cross_max,'or'); hold off;

length_bubble = loc_mean - ini_bubble; 
length_bubble_prev = end_bubble - ini_bubble;
N_bubbles = length(length_bubble);
Mean_freq = N_bubbles/(Time(end) - Time(1));

%% Estimation process 
% Let's assume that the volume dispensed in 10 seconds, if we multiply by 6
% to be in minutes, must be 10% deviated from the pumping volume 
error = 0.01; 
dexcen = [0.1; 0.05; 0.01; 0.005]; 
loops = zeros(length(dexcen),1); 
Mean_volumes = []; Mean_minor_axis = [];
Volume_objective = Water_pump*60;
Min_volume = Water_pump*10*(1+error);
Max_volume = Water_pump*10*(1-error);

for  ii = 1:length(dexcen)
    excen = excen0;
    % Estimated minor axis
    Minor_axis_vec = sqrt(1-excen0^2)*(length_bubble/F_sampling*Fluid_vel)/2;

    % Ellipsoidal Cilindrical bubbles: 
    % Volume = pi* L_bubble (um) / 2* Minor_axis (um) * Lbubble (um) * 1E-9 = 
    % pi* L_bubble (um) / 2 * Minor_axis (um) * (Tbubble(s) * vel (um/s)) (1E-9 uL) 
    Volume = pi * (((length_bubble/F_sampling)*...
        Fluid_vel)).* Minor_axis_vec * Channel_depth * 1E-9 / 2;
    Mean_volume_10s = pi * sum(((length_bubble)/F_sampling)*...
        Fluid_vel .* Minor_axis_vec) * Channel_depth * 1E-9 / 2;

    Mean_minor_axis = [Mean_minor_axis; sum(Minor_axis_vec)/length(Minor_axis_vec)];
    Mean_volumes = [Mean_volumes; Mean_volume_10s*6];
    
    while (Mean_volume_10s > Min_volume || Mean_volume_10s < Max_volume) ...
            && any(Minor_axis_vec < (Channel_width/2)) && excen > dexcen(ii) ...
            && excen < 1-dexcen(ii) && loops(ii) < 5000 
        % Estimated minor axis, if the calculated volume is less than the pump
        % volume, minor axis must increase (eccentrictiy decreases). 
        % On the contrary, must decrease (eccentricity increases).
        % Excentricity = 0 --> Minor_axis = Major_axis (Circle)
        loops(ii) = loops(ii) + 1;
        excen = excen + sign(Mean_volume_10s-Volume_objective/6)*dexcen(ii); 
        excen = min(excen,1);
        Minor_axis_vec = sqrt(1-excen^2)*(length_bubble/F_sampling*Fluid_vel)/2; 

        % Ellipsoidal Cilindrical bubbles: 
        % Volume = pi* L_bubble (um) / 2* Minor_axis (um) * Lbubble (um) * 1E-9 = 
        % pi* L_bubble (um) / 2 * Minor_axis (um) * (Tbubble(s) * vel (um/s)) (1E-9 uL) 
        Volume = pi * (((length_bubble/F_sampling)*...
            Fluid_vel)).* Minor_axis_vec * Channel_depth * 1E-9 / 2;
        Mean_volume_10s = pi * sum(((length_bubble)/F_sampling)*...
            Fluid_vel .* Minor_axis_vec) * Channel_depth * 1E-9 / 2; 

        Mean_minor_axis = [Mean_minor_axis; sum(Minor_axis_vec)...
            /length(Minor_axis_vec)];
        Mean_volumes = [Mean_volumes; Mean_volume_10s*6];
    end
end
%% Plots

figure('Name','Efficiency')
pos = 1; hold on
for ii = 1:length(dexcen)
    subplot(211)
    plot((Mean_volumes(pos:pos+loops(ii))-Volume_objective)/Volume_objective*100,...
        '-*','DisplayName', ['de = ', num2str(dexcen(ii))]);
    hold on
    subplot(212)
    plot((Mean_minor_axis(pos:pos+loops(ii))-(560/2))/(560/2)*100,...
        '-*','DisplayName', ['de = ', num2str(dexcen(ii))]);
    hold on
    pos = pos + loops(ii) + 1;
end
subplot(211)
ylabel('Relative error (%)')
title('Mean volume')
legend('show')
subplot(212)
ylabel('Relative error (%)')
title('Mean minor axis')
legend('show')
hold off

Major_axis = 0.5*(sum(length_bubble)/N_bubbles/F_sampling)*Fluid_vel;
Minor_axis = Mean_minor_axis(end); lim = max([Major_axis,Channel_width/2]);
N_grid = 1000;
% xx = linspace(-Channel_depth,Channel_depth,N_grid)*0.5; % Depth channel
zz = linspace(-Major_axis,Major_axis,N_grid)'; % Length channel
yy = sqrt(1-(zz/Major_axis).^2)*Minor_axis; % Width channel
zzt = linspace(-413.9412,413.9412,N_grid)'; % Calculated with Fittings.m
yyt = sqrt(1-(zzt/413.9412).^2)*275.313; % Calculated with Fitting.m

figure('Name','Bubble shape')
plot([-yy; yy],[zz; zz],'-r','DisplayName','Estimated'); hold on
plot([-yyt; yyt],[zzt; zzt],'-g','DisplayName','Analyzed');
line(Channel_width/2*[1 1], lim*[-1 1])
line(Channel_width/2*[-1 -1], lim*[-1 1]); hold off
xlabel 'Width(um)'
ylabel 'Length(um)'
axis([-lim lim -lim lim])
pbaspect([1 1 1])
legend('Estimated','Analyzed')

figure('Name','Length of bubbles and volume')
suptitle(['Excentricity = ', num2str(excen)])
subplot(131)
plot(length_bubble/F_sampling*Fluid_vel,'*r'); hold on
plot(Minor_axis_vec,'ob'); hold off
legend('Major axis','Minor axis')
ylabel micrometers

subplot(132)
plot(Volume,'*r')
ylabel microLiters

subplot(133)
plot(Volume*N_bubbles*6,'*r')
ylabel microLiters/min

figure('Name','Data')
subplot(211)
plot(Time,Data); hold on
plot([0 Time(end)],threshold*[1 1],'-k'); 
plot(loc_min/F_sampling, cross_min,'*r'); 
plot(loc_max/F_sampling, cross_max,'*r'); 
plot(loc_mean/F_sampling, (cross_min+cross_max)/2,'dr'); 
hold off;
xlabel('Time (s)')
ylabel('Power variation')
subplot(212)
plot(Time,Digital_data,'-r'); hold on
plot(Time,Digital_data_prev,'-g');
plot(Time(2:end),Digital_data_derv/max(Digital_data_derv),'-b'); hold off
xlabel('Time (s)')
legend('Digital signal 2 peaks', 'Digital signal 1 peak', 'Derivative')

%% FFT of the signal
N_fft = 2^floor(log2(N_samples)); % Number of freqs for the FFT
FFT_freqs = F_sampling*(0:(N_fft/2))'/N_fft;
FFT_vec = fft(Data,N_fft); FFT_vec(N_fft/2 + 2:end) = [];
isearch = round((Mean_freq-0.5)*N_fft/F_sampling);
iwindow = round(1*N_fft/F_sampling);
[~, imax] = max(abs(FFT_vec(isearch:isearch + iwindow))); 
imax = imax + isearch - 1;
Bubble_freq = FFT_freqs(imax);
disp(['Mean frequency (FFT) : ', num2str(Bubble_freq), ' Hz'])

figure('Name','FFT')
plot(FFT_freqs, abs(FFT_vec),'-b'); hold on
plot(Bubble_freq, abs(FFT_vec(imax)),'*r'); hold off
xlabel('Frequency (Hz)')
ylabel('PSD')