clear all
close all

%% Parameters
N = 101; % Number of rows. N = M square lattice ( x = y )
N_samples = 10E3; % Number of samples to read
F_sampling = 200E3; % Sampling frequency of the DAQ
F_piezo = 300; % Frequency of the harmonic motion
N_fft = 2^floor(log2(N_samples)); % Number of freqs for the FFT
FFT_freqs = F_sampling*(0:(N_fft/2))'/N_fft;

N_fringes = 17; T_fringes = 1E-3; % Number of fringes in a time frame
F_search = N_fringes/T_fringes; % Freq of the fringes = interested in
isearch = round(F_search*N_fft/F_sampling); % Position in the FFT_freqs of F_search
iband = round(1E3*N_fft/F_sampling); % Window constrain

load_loop_data = 0; % 0 = Load data, otherwise NO
filtering = 0; % 0 = Filter data, otherwise NO
iiplot = (N - 1)/2; % Y position of the 2D matrix to plot
jjplot = iiplot; % X position of the 2D matrix to plot

%% Import data
if load_loop_data == 0
    Folder = 'full_scan_3x3mm_v4\'; 
    PD_LD = zeros(N,N,N_samples); PD_mobile = PD_LD; PD_fixed = PD_LD;
    LD_voltage = PD_LD; Stimulus = PD_LD;
    for ii = 1:N
        Subfolder = ['x',num2str(ii-1)];
        for jj = 1:N
            File_name = [Folder, Subfolder,'\y',num2str(jj-1)];
            Load_data = load(File_name);
            PD_LD(N-jj+1,ii,:) = zscore(Load_data(:,2))...
                /max(abs(zscore(Load_data(:,2))));
            PD_mobile(N-jj+1,ii,:) = zscore(Load_data(:,3))...
                /max(abs(zscore(Load_data(:,3))));
            PD_fixed(N-jj+1,ii,:) = zscore(Load_data(:,4))...
                /max(abs(zscore(Load_data(:,4))));
            LD_voltage(N-jj+1,ii,:) = -zscore(Load_data(:,5))...
                /max(abs(zscore(Load_data(:,5)))); % It is inverted (-1 gain)
            Stimulus(N-jj+1,ii,:) = zscore(Load_data(:,6))...
                /max(abs(zscore(Load_data(:,6))));
        end
    end
    Time = Load_data(:,1); 
    save('Full_scan_3x3_v4.mat','PD_LD','PD_mobile','PD_fixed','LD_voltage','Stimulus','Time')
else
    load('Full_scan_3x3_v4.mat');
    % Time = (0:1/F_sampling:(N_samples-1)/F_sampling)';
end
disp(['Data section done. The sampling frequency selected is ',...
    num2str(F_sampling),'Hz, whereas the sampling frequency calculated ',...
    'based on the Load data is ', num2str(1/(Time(2)-Time(1))), 'Hz.'])

%% Filter
if filtering == 0
    %% Filter design
    % Fstop = Frequency at the end of the stop band (in pi/samples)
    % Fpass = Frequency at the start of the pass band
    
    N_filter = 1000; % Order of the filter (even better)
    Fstop = 700; Fpass = 1E3; 
    Wstop = 100; Wpass = 100; % Weighting for Fstop and Fpass
    
    BPF = firpm(N_filter, [0 Fstop Fpass F_sampling/2]/(F_sampling/2),...
        [0 0 1 1], [Wstop Wpass]);
    
    fvtool(BPF,1); 
    grpdelay(BPF,N_samples,F_sampling); samples_delay = mean(grpdelay(BPF));
    
    figure('Name','Filter signal')
    subplot(221)
    plot(Time, reshape(PD_LD(iiplot,jjplot,:),N_samples,1),'-b'); hold on;
    plot(Time, reshape(PD_mobile(iiplot,jjplot,:),N_samples,1),'-r');
    plot(Time, reshape(PD_fixed(iiplot,jjplot,:),N_samples,1),'-g');
    plot(Time, reshape(LD_voltage(iiplot,jjplot,:),N_samples,1),'-k');
    xlabel('Time (s)')
    ylabel('Power variation (a.u.)')
    legend('Back PD','Scanning PD','Fixed PD','LD Inverted Voltage','Location','best');
    hold off
    
    subplot(222)
    fftvec = reshape(abs(fft(PD_LD(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-b'); hold on;
    fftvec = reshape(abs(fft(PD_mobile(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-r');
    fftvec = reshape(abs(fft(PD_fixed(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-g');
    fftvec = reshape(abs(fft(LD_voltage(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-k');
    xlabel('Frequency (Hz)')
    ylabel PSD
    legend('Back PD','Scanning PD','Fixed PD','LD Inverted Voltage','Location','best');
    hold off

    %% Apply the filter
    for ii=1:N
        for jj=1:N
            PD_LD(ii,jj,:) = filter(BPF,1,PD_LD(ii,jj,:));
            PD_mobile(ii,jj,:) = filter(BPF,1,PD_mobile(ii,jj,:)); 
            PD_fixed(ii,jj,:) = filter(BPF,1,PD_fixed(ii,jj,:));
            LD_voltage(ii,jj,:) = filter(BPF,1,LD_voltage(ii,jj,:));
        end
    end
    
    % Chop the signals due to the delay
    PD_LD(:,:,1:samples_delay) = [];
    PD_mobile(:,:,1:samples_delay) = [];
    PD_fixed(:,:,1:samples_delay) = [];
    LD_voltage(:,:,1:samples_delay) = [];
    Time = Time(1:end-samples_delay); 
    Stimulus = Stimulus(1:end-samples_delay);
    N_samples = N_samples - samples_delay;
    
    subplot(223)
    plot(Time, reshape(PD_LD(iiplot,jjplot,:),N_samples,1),'-b'); hold on;
    plot(Time, reshape(PD_mobile(iiplot,jjplot,:),N_samples,1),'-r');
    plot(Time, reshape(PD_fixed(iiplot,jjplot,:),N_samples,1),'-g');
    plot(Time, reshape(LD_voltage(iiplot,jjplot,:),N_samples,1),'-k');
    legend('Back PD','Scanning PD','Fixed PD','LD Inverted Voltage','Location','best');
    xlabel('Time (s)')
    ylabel('Power variation (a.u.)')
    hold off
    
    subplot(224)
    fftvec = reshape(abs(fft(PD_LD(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-b'); hold on;
    fftvec = reshape(abs(fft(PD_mobile(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-r');
    fftvec = reshape(abs(fft(PD_fixed(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-g');
    fftvec = reshape(abs(fft(LD_voltage(iiplot,jjplot,:),N_fft)),N_fft,1);
    plot(FFT_freqs, fftvec(1:N_fft/2 + 1),'-k');
    xlabel('Frequency (Hz)')
    ylabel PSD
    legend('Back PD','Scanning PD','Fixed PD','LD Inverted Voltage','Location','best');
    hold off    
    
end
disp('Filtering section done')

%% Phase calculus
PD_LD_phase = zeros(N,N,1); PD_mobile_phase = PD_LD_phase; 
PD_fixed_phase = PD_LD_phase; LD_voltage_phase = PD_LD_phase;

PD_LD_rms = PD_LD_phase; PD_mobile_rms = PD_LD_phase; 
PD_fixed_rms = PD_LD_phase; LD_voltage_rms = PD_LD_phase;

Mobile_lag = zeros(N,N); Fixed_lag = Mobile_lag; Voltage_lag = Mobile_lag;
for ii = 1:N
    for jj = 1:N
        %% RMS Intensity determination
        PD_LD_rms(ii,jj) = rms(PD_LD(ii,jj,:));
        PD_mobile_rms(ii,jj) = rms(PD_mobile(ii,jj,:));
        PD_fixed_rms(ii,jj) = rms(PD_fixed(ii,jj,:));
        LD_voltage_rms(ii,jj) = rms(LD_voltage(ii,jj,:));
        
        %% Phase determination
        % LD Pd phase
        fftvec_ld = reshape(fft(PD_LD(ii,jj,:),N_fft),N_fft,1); % FFT
        fftvec_ld = fftvec_ld(1:N_fft/2 + 1); % Half of the spectrum 
        [~,imax] = max(abs(fftvec_ld(isearch:isearch+iband)));
        imax = imax + isearch - 1;
        % Find the freq of the maximum peak
        PD_LD_phase(ii,jj) = ...
            mod(atan2d(imag(fftvec_ld(imax)),real(fftvec_ld(imax)))+360,360);
            % Get the phase [-180,180] of the maximum peak and convert it 
            % into a [0,360] phase
            
        % Mobile Pd phase
        fftvec_pdm = reshape(fft(PD_mobile(ii,jj,:),N_fft),N_fft,1); % FFT
        fftvec_pdm = fftvec_pdm(1:N_fft/2 + 1); % Half of the spectrum
        [~,imax] = max(abs(fftvec_pdm(isearch:isearch+iband)));
        imax = imax + isearch - 1;
        % Find the freq of the maximum peak
        PD_mobile_phase(ii,jj) = ...
            mod(atan2d(imag(fftvec_pdm(imax)),real(fftvec_pdm(imax)))+360,360);
            % Get the phase [-180,180] of the maximum peak and convert it 
            % into a [0,360] phase
        
        % Fixed Pd phase
        fftvec_pdf = reshape(fft(PD_fixed(ii,jj,:),N_fft),N_fft,1); % FFT
        fftvec_pdf = fftvec_pdf(1:N_fft/2 + 1); % Half of the spectrum
        [~,imax] = max(abs(fftvec_pdf(isearch:isearch+iband)));
        imax = imax + isearch - 1;
        % Find the freq of the maximum peak
        PD_fixed_phase(ii,jj) = ...
            mod(atan2d(imag(fftvec_pdf(imax)),real(fftvec_pdf(imax)))+360,360); 
            % Get the phase [-180,180] of the maximum peak and convert it 
            % into a [0,360] phase
            
        % LD voltage phase
        fftvec_LDV = reshape(fft(LD_voltage(ii,jj,:),N_fft),N_fft,1); % FFT
        fftvec_LDV = fftvec_LDV(1:N_fft/2 + 1); % Half of the spectrum
        [~,imax] = max(abs(fftvec_LDV(isearch:isearch+iband)));
        imax = imax + isearch - 1;
        % Find the freq of the maximum peak
        LD_voltage_phase(ii,jj) = ...
            mod(atan2d(imag(fftvec_LDV(imax)),real(fftvec_LDV(imax)))+360,360); 
            % Get the phase [-180,180] of the maximum peak and convert it 
            % into a [0,360] phase
        
        %% IN/OUT of phase determination through Cross-correlation
        [corr,lag] = xcorr(reshape(PD_LD(ii,jj,:),N_samples,1),...
            reshape(PD_mobile(ii,jj,:),N_samples,1));
            % Cross-correlate the mobile PD with the back LD PD
        corr = corr(lag>0); lag = lag(lag>0); 
        corr(corr<0) = 0; [~,imax] = max(corr); 
        Mobile_lag(ii,jj) = mod((lag(imax)/F_sampling)+(1/F_piezo),1/F_piezo);
            % Find the maximum value of the cross-correlation and divide
            % its lag by the sampling frequency
        
        [corr,lag] = xcorr(reshape(PD_LD(ii,jj,:),N_samples,1),...
            reshape(PD_fixed(ii,jj,:),N_samples,1));
            % Cross-correlate the fixed PD with the back LD PD
        corr = corr(lag>0); lag = lag(lag>0); 
        corr(corr<0) = 0; [~,imax] = max(corr); 
        Fixed_lag(ii,jj) = lag(imax)/F_sampling;
            % Find the maximum value of the cross-correlation and divide
            % its lag by the sampling frequency
            
        [corr,lag] = xcorr(reshape(PD_LD(ii,jj,:),N_samples,1),...
            reshape(LD_voltage(ii,jj,:),N_samples,1));
            % Cross-correlate the fixed PD with the back LD PD
        corr = corr(lag>0); lag = lag(lag>0); 
        corr(corr<0) = 0; [~,imax] = max(corr); 
        Voltage_lag(ii,jj) = lag(imax)/F_sampling;
            % Find the maximum value of the cross-correlation and divide
            % its lag by the sampling frequency. However, as the signal was
            % inverted, we should take it into account when making
            % conclusions.
        
    end
end
disp('Phase section done')

%% Plot the intensity graphs
%%%%%%%%%%%%%%%%%%%%%%%%%%%% INTENSITY %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure('Name','Intensity')
subplot(221)
imagesc(PD_LD_rms)
cmap = jet(256); % 256 indexed colors
pbaspect([1 1 1])
colormap(cmap);
colorbar
title('LD RMS')
xlabel X
ylabel Y

subplot(222)
imagesc(PD_mobile_rms)
pbaspect([1 1 1])
colorbar
title('Mobile RMS')
xlabel X
ylabel Y

subplot(223)
imagesc(PD_fixed_rms)
pbaspect([1 1 1])
colorbar
title('Fixed RMS')
xlabel X
ylabel Y

subplot(224)
imagesc(LD_voltage_rms)
pbaspect([1 1 1])
colorbar
title('Inverted Voltage RMS')
xlabel X
ylabel Y

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% PHASE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure('Name','Phase')
subplot(221)
imagesc(PD_LD_phase)
pbaspect([1 1 1])
colormap(cmap)
colorbar
caxis([0 360])
title('PD from the LD')
xlabel X
ylabel Y

subplot(222)
imagesc(PD_mobile_phase)
pbaspect([1 1 1])
colorbar
caxis([0 360])
title('Scanning PD')
xlabel X
ylabel Y

subplot(223)
imagesc(PD_fixed_phase)
pbaspect([1 1 1])
title('Fixed PD')
colorbar
caxis([0 360])
xlabel X
ylabel Y

subplot(224)
imagesc(LD_voltage_phase)
pbaspect([1 1 1])
title('LD Inverted Voltage')
colorbar
caxis([0 360])
xlabel X
ylabel Y

%%%%%%%%%%%%%%%%%%%%%%%%%%%%% IN/OUT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In or out of phase?
figure('Name', 'IN/OUT phase')
subplot(321) % Absolute value of the difference
imagesc(abs(PD_mobile_phase - PD_LD_phase))
pbaspect([1 1 1])
axmap1 = gca;
jet_wrap = jet(256);
jet_wrap = vertcat(jet_wrap,flipud(jet_wrap)); % Make the colorbar circular
colormap(axmap1, jet_wrap);
c = colorbar;
caxis([0 360]) 
set(c,'Ticks',[ 0, 180, 360],...
    'TickLabels',{'IN','OUT','IN'})
title('|Phase_{Scan} - Phase_{LD}|')

subplot(322) % Lag calculated with the cross-correlation
imagesc(Mobile_lag)
pbaspect([1 1 1])
axmap2 = gca;
colormap(gca, jet_wrap);
c = colorbar;
caxis([0 1/F_piezo])
set(c,'Ticks',[0, 0.5/F_piezo, 1/F_piezo],...
         'TickLabels',{'IN','OUT','IN'})
title('Cross-correlation Scan - LD')

subplot(323) % Absolute value of the difference
imagesc(abs(PD_fixed_phase - PD_LD_phase))
pbaspect([1 1 1])
axmap3 = gca;
colormap(axmap3, jet_wrap)
c3 = colorbar;
caxis([0 360])
set(c3,'Ticks',[ 0, 180, 360],...
    'TickLabels',{'IN','OUT','IN','OUT','IN'})
title('|Phase_{Fixed} - Phase_{LD}|')

subplot(324) % Lag calculated with the cross-correlation
imagesc(Fixed_lag)
pbaspect([1 1 1])
axmap4 = gca;
colormap(axmap4, jet_wrap)
c4 = colorbar;
caxis([0 1/F_piezo])
set(c4,'Ticks',[0, 0.5/F_piezo, 1/F_piezo],...
         'TickLabels',{'IN','OUT','IN'})
title('Cross-correlation Fixed - LD')

subplot(325) % Absolute value of the difference
imagesc(abs(LD_voltage_phase - PD_LD_phase))
pbaspect([1 1 1])
axmap5 = gca;
colormap(axmap5, jet_wrap)
c5 = colorbar;
caxis([0 360])
set(c5,'Ticks',[ 0, 180, 360],...
    'TickLabels',{'IN','OUT','IN','OUT','IN'})
title('|Phase_{Inverted Voltage} - Phase_{LD}|')

subplot(326) % Lag calculated with the cross-correlation
imagesc(Voltage_lag)
pbaspect([1 1 1])
axmap6 = gca;
colormap(axmap6, jet_wrap)
c6 = colorbar;
caxis([0 1/F_piezo])
set(c6,'Ticks',[0, 0.5/F_piezo, 1/F_piezo],...
         'TickLabels',{'IN','OUT','IN'})
title('Cross-correlation Inverted Voltage - LD')
