clear all
close all

%% Parameters
Lambda = 1.550; % 1550 nm LASER
Range = 2; % Number of lambda ranges to explore
Zab_res = 0.047625; % Resolution in microm/step
Zab_steps = 1; 
Start_pos = 25000; % 25000 microm
N_points = floor(Range*Lambda/Zab_res); 
N_loops = 4;
Start_time = datenum([2017,04,10,17,45,37]);
End_time = datenum([2017,04,11,4,41,59]);
% N_points = 41; % Phase_Data_04_04_2017 had a bad N_points calculus
L_vec = (Start_pos:Zab_res*Zab_steps:...
    Start_pos+Lambda*Range-Zab_res*Zab_steps)'; % The last position NO

%% Import data
m_avail = 1; L = 0;

m_avail_str = ['001'; '002'; '003'; '004'; '005'; '006'; '007'; '008'; ...
    '009'; '010'; '011'; '012'; '013'; '014'; '015'; '016'; '017'; '018'; ...
    '019'; '020'; '021'; '022'; '023'; '024'; '025'; '026'; '027'; '028'];
for ii = 1:N_loops
    for jj = 1:N_points
        if round(L_vec(jj)) == L
            m_avail = m_avail + 1;
        else
            m_avail = 1;
        end
        L = round(L_vec(jj));
        Folder = 'Phase_Data_10_04_2017\';
        File_name = [Folder,'Phase_Data_',num2str(L),'um_Loop_'...
            num2str(ii-1),'_',m_avail_str(m_avail,:)];
        Load_data = load(File_name);
        PD_LD(:,jj,ii) = zscore(Load_data(:,2))/max(abs(zscore(Load_data(:,2)))); 
        PD_EXT(:,jj,ii) = zscore(Load_data(:,4))/max(abs(zscore(Load_data(:,4))));
    end
end
Abs_time = linspace(Start_time,End_time,N_points*N_loops)';
Time = Load_data(1); 
Phase_vec = load([Folder,'Phase_Array.lvm']);
    
%% Calculate the mean 
phase = zeros(N_points,N_loops); phase_mean = zeros(N_points,1);
for jj = 1:N_points
    for ii = 1:N_loops
    [~,phase(jj,ii)] = fit_ellipse(PD_LD(:,jj,ii),PD_EXT(:,jj,ii));
    end
    phase_mean(jj) = sum(phase(jj,:))/N_loops;
end
phase = phase*(180/pi); phase_mean = phase_mean*(180/pi);

%% Plot the mean
figure('Name','Phase')
kk = 1;
while kk < N_loops + 1
    for ii = 1:N_points
        plot(L_vec(ii)-Start_pos,phase(ii,1:kk),'*')
        hold on
        phase_mean(ii) = sum(phase(ii,1:kk))/kk;
    end
    plot(L_vec-Start_pos,phase_mean,'-^')
    hold off
    title(['Number of loops: ',num2str(kk)])
    ylabel Phase(45=IN/-45=OUT)
    xlabel L(micrometers)
    kk = kk + 1; pause 
end

figure('Name','Phase vs time')
N_plots = 4; ii = 1;
for jj = 1:N_plots
    subplot(N_plots,1,jj)
    plot(Abs_time((ii:N_points:ii+N_points*(N_loops-1))),phase(ii,:),'-*')
    datetick('x',13,'keepticks')
    ylabel(['L = ', num2str(L_vec(ii)-Start_pos)])
    ii = ii + round(N_points/N_plots);
end

figure('Name','XY graph vs time')
N_plots = 4; dist = 10;
for jj = 1:N_plots
    subplot(N_plots/2,2,jj)
    [~,phase(dist,jj)] = fit_ellipse(PD_LD(:,dist,jj),PD_EXT(:,dist,jj));
    xlabel PD_{LD}
    ylabel PD_{EXTERNAL}
    title(['L = ',num2str(L_vec(dist)-Start_pos),'. T = ', ...
        datestr(Abs_time(jj + N_points*(jj-1)))])
    grid on
end

Abs_phase = phase(:);
figure('Name','Whole phase vs time')
n_loops_print = 4;
end_pos = n_loops_print*N_points;
plot(Abs_time(1:end_pos), Abs_phase(1:end_pos),'*')
hold on
for jj = 1:n_loops_print-1
    plot([Abs_time(N_points*jj) Abs_time(N_points*jj)],...
        [-45 45],'-k')
end
title(['Number of loops: ',num2str(n_loops_print)])
ylabel Phase(45=IN/-45=OUT)
datetick('x',13,'keepticks')

%% Plot data
N_fig = Range*2; % Each lambda/2 in a figure
N_plots = floor(N_points/N_fig); % Number of plots for a same row

if N_fig*N_plots < N_points
    N_plots = ones(N_fig,1)*N_plots;
    N_plots(end) = N_plots(end) + 1;
end

figure('Name','Phase')
plot(L_vec-Start_pos,phase,'*b')
title(['Number of loops: ',num2str(n_loops_print)])
ylabel Phase(45=IN/-45=OUT)
xlabel L(micrometers)
